mod metar_tab;

use std::collections::HashMap;

// Strum Package to make enum Itterable
use strum::EnumIter;
use strum::IntoEnumIterator;

use self::metar_tab::MetarTab;

#[derive(Clone, Copy, Debug, Default, EnumIter, Eq, Hash, PartialEq)]
pub enum TabTypes {
    #[default]
    Home,
    METAR,
    TAF,
    NOTAMS,
    Routing,
}

pub struct TabState {
    current_tab: TabTypes,
    tabs: HashMap<TabTypes, Box<dyn Tab>>,
}

impl Default for TabState {
    fn default() -> Self {
        let mut ts = Self {
            current_tab: TabTypes::default(),
            tabs: HashMap::new(),
        };

        // Add each tab to the map
        ts.tabs
            .insert(TabTypes::METAR, Box::new(MetarTab::default()));

        ts
    }
}

impl TabState {
    /// Get the Tab based on a given Tab Type
    pub fn get_tab_state(&mut self, tab_type: TabTypes) -> Option<&mut Box<dyn Tab>> {
        match self.tabs.get_mut(&tab_type) {
            Some(tab) => Some(tab),
            None => None,
        }
    }

    /// Get the currently selected Tab
    pub fn get_current_tab(&mut self) -> Option<&mut Box<dyn Tab>> {
        self.get_tab_state(self.current_tab)
    }

    /// Get the type of the current Tab
    pub fn get_current_tab_type(&self) -> TabTypes {
        return self.current_tab;
    }

    /// Change the current tab to a different tab type, saving the state of the current tab
    pub fn change_tab(&mut self, new_tab_type: &TabTypes) {
        self.current_tab = *new_tab_type;
    }
}

/// Tab traits that allow tab display on the main application window
pub trait Tab {
    async fn display_tab(&mut self, ctx: &egui::Context, frame: &egui::Frame);
}

/// Function to draw main set of tabs on the window
pub fn display_tabs(tab_state: &mut TabState, ctx: &egui::Context, frame: &egui::Frame) {
    egui::TopBottomPanel::top("app_top_bar")
        .frame(*frame)
        .show(ctx, |ui| {
            ui.horizontal_wrapped(|ui| {
                ui.visuals_mut().button_frame = false;

                // Insert Tabs
                for tab in TabTypes::iter() {
                    let tab_label = ui.selectable_label(
                        tab == tab_state.get_current_tab_type(),
                        format!("{:?}", tab),
                    );

                    if tab_label.clicked() {
                        tab_state.change_tab(&tab);
                    }
                }
            });
        });
}
